const Promise = require("bluebird");
var request = Promise.promisifyAll(require("request"));
const jstoxml = require('jstoxml');
const merge = require('lodash.merge');
const xml2js = require("xml2js").parseString;

request = request.defaults({jar: true});

var defaults = {
	realm: 'www',
	domain: 'quickbase.com',
	path: 'db',
	proxy: '',

	username: '',
	password: '',
	appToken: '',
	userToken: '',
	dbid: 'main',
	ticket: ''

}


class QuickBase {
	// 
	constructor(options) {
			this.className = 'QuickBase';
			this.settings = merge({}, defaults, options || {});
	}
	
	api(api, options, callback){
		var process = new processAll(this, api, options, callback);
		return process
			.processBody()
			.requestOptions()
			.sendRequest()
	}
	
	connect(){
		if(this.settings.username && this.settings.password && !this.settings.ticket){
			var setTicket = new processAll(this, "API_Authenticate")
			return setTicket
				.processBody()
				.requestOptions()
				.sendRequest()
		} else {
			return new Promise((resolve, reject) => {
				resolve();
			});
		}
	}
	
}

class processAll {
	constructor(parent, action, options, callback) {
		this.parent = parent;
		this.action = action;
		this.options = merge({}, options || {});
		this.callback = callback;
		this.settings = parent.settings;
		return this;
	}
	
	processBody(){
		if(!this.options.dbid){
			this.dbid = this.settings.dbid;
		} else {
			this.dbid = this.options.dbid;
		}
		if(this.action === 'API_Authenticate'){
			this.options.username = this.settings.username;
			this.options.password = this.settings.password;
		} else if(this.dbid !== 'main' && this.settings.userToken && !this.settings.ticket){
			this.options.usertoken = this.settings.usertoken;
		} else {
			this.options.ticket = this.settings.ticket;
		}
		if(this.settings.appToken){this.options.apptoken = this.settings.appToken}
		if(this.options.records_csv){this.options.records_csv = '<![CDATA[' + this.options.records_csv + ']]>';}
		this.body = {};
		delete this.options.dbid;
		if(typeof(this.options.qdbapi) === 'undefined'){
			this.body.qdbapi = this.options;
			this.xml = jstoxml.toXML(this.body,{header: false, indent: '  '});
		} else {
			this.body.qdbapi = this.options.qdbapi;
			this.xml = jstoxml.toXML(this.body,{header: false, indent: '  '});
		}
		delete this.options;
		return this;
	}

	requestOptions(){
		this.reqOps = {
			method: "POST",
			headers: {
				'Content-Type': 'application/xml',
				'QUICKBASE-ACTION': this.action
			},
			uri:'https://' +  this.settings.realm + "." + this.settings.domain + "/" + this.settings.path + "/" + this.dbid,
			proxy: this.settings.proxy,
			body: this.xml
		}
		return this;
	}
	
	sendRequest(){
		return new Promise((resolve, reject) => {
			request(this.reqOps,(error, response, bod) => {
				if (this.callback instanceof Function) {
					callback(null, bod);
				} else {
					if(this.action === "API_Authenticate"){
						var t = /(?:<ticket>)(.*?)(?:<\/ticket>)/i.exec(bod);
						this.settings.ticket = t[1];
						console.log(this.settings.ticket);
						resolve(bod);
					} else {
						resolve(bod);
					}					
				}
			});
		});
	}
}

if (typeof module !== 'undefined' && module.exports) {
	module.exports = QuickBase;
}